@@ -3,19 +3,18 @@ use std::convert::TryInto;
 use std::result::Result;
 use std::str::FromStr;
 
+use bat::assets::HighlightingAssets;
 use console::Term;
-use structopt::clap;
 
 use crate::cli;
 use crate::config;
-use crate::env;
+use crate::env::DeltaEnv;
 use crate::errors::*;
 use crate::fatal;
 use crate::features;
 use crate::git_config::{GitConfig, GitConfigEntry};
 use crate::options::option_value::{OptionValue, ProvenancedOptionValue};
 use crate::options::theme;
-use crate::utils::bat::assets::HighlightingAssets;
 use crate::utils::bat::output::PagingMode;
 
 macro_rules! set_options {
@@ -25,7 +24,7 @@ macro_rules! set_options {
         $(
             let kebab_case_field_name = stringify!($field_ident).replace("_", "-");
             let option_name = $expected_option_name_map[kebab_case_field_name.as_str()];
-            if !$crate::config::user_supplied_option(&option_name, $arg_matches) {
+            if !$crate::config::user_supplied_option(&kebab_case_field_name, $arg_matches) {
                 if let Some(value) = $crate::options::get::get_option_value(
                     option_name,
                     &$builtin_features,
@@ -45,11 +44,13 @@ macro_rules! set_options {
                 "diff-highlight", // Does not exist as a flag on config
                 "diff-so-fancy", // Does not exist as a flag on config
                 "features",  // Processed differently
+                "help", // automatically added by clap
                 // Set prior to the rest
                 "no-gitconfig",
                 "dark",
                 "light",
                 "syntax-theme",
+                "version", // automatically added by clap
             ]);
             let expected_option_names: HashSet<_> = $expected_option_name_map.values().cloned().collect();
 
@@ -75,12 +76,12 @@ pub fn set_options(
         }
         set_git_config_entries(opt, git_config);
     }
-    opt.navigate = opt.navigate || env::get_boolean_env_var("DELTA_NAVIGATE");
+    opt.navigate = opt.navigate || opt.env.navigate.is_some();
     if opt.syntax_theme.is_none() {
-        opt.syntax_theme = env::get_env_var("BAT_THEME")
+        opt.syntax_theme = opt.env.bat_theme.clone();
     }
 
-    let option_names = cli::Opt::get_option_names();
+    let option_names = cli::Opt::get_argument_and_option_names();
 
     // Set features
     let mut builtin_features = features::make_builtin_features();
@@ -125,8 +126,11 @@ pub fn set_options(
 
     set_options!(
         [
+            blame_code_style,
             blame_format,
+            blame_separator_format,
             blame_palette,
+            blame_separator_style,
             blame_timestamp_format,
             color_only,
             commit_decoration_style,
@@ -140,6 +144,7 @@ pub fn set_options(
             file_modified_label,
             file_removed_label,
             file_renamed_label,
+            file_regex_replacement,
             right_arrow,
             hunk_label,
             file_style,
@@ -331,7 +336,7 @@ fn gather_features(
     builtin_features: &HashMap<String, features::BuiltinFeature>,
     git_config: &Option<GitConfig>,
 ) -> Vec<String> {
-    let from_env_var = env::get_env_var("DELTA_FEATURES");
+    let from_env_var = &opt.env.features;
     let from_args = opt.features.as_deref().unwrap_or("");
     let input_features: Vec<&str> = match from_env_var.as_deref() {
         Some(from_env_var) if from_env_var.starts_with('+') => from_env_var[1..]
@@ -458,12 +463,9 @@ fn gather_builtin_features_from_flags_in_gitconfig(
     git_config: &GitConfig,
 ) {
     for child_feature in builtin_features.keys() {
-        if let Some(value) =
-            git_config.get::<bool>(&format!("{}.{}", git_config_key, child_feature))
+        if let Some(true) = git_config.get::<bool>(&format!("{}.{}", git_config_key, child_feature))
         {
-            if value {
-                gather_builtin_features_recursively(child_feature, features, builtin_features, opt);
-            }
+            gather_builtin_features_recursively(child_feature, features, builtin_features, opt);
         }
     }
 }
@@ -502,17 +504,15 @@ fn gather_builtin_features_recursively(
         }
         for child_feature in builtin_features.keys() {
             if let Some(child_features_fn) = feature_data.get(child_feature) {
-                if let ProvenancedOptionValue::DefaultValue(OptionValue::Boolean(value)) =
+                if let ProvenancedOptionValue::DefaultValue(OptionValue::Boolean(true)) =
                     child_features_fn(opt, &None)
                 {
-                    if value {
-                        gather_builtin_features_recursively(
-                            child_feature,
-                            features,
-                            builtin_features,
-                            opt,
-                        );
-                    }
+                    gather_builtin_features_recursively(
+                        child_feature,
+                        features,
+                        builtin_features,
+                        opt,
+                    );
                 }
             }
         }
@@ -640,7 +640,7 @@ fn set_true_color(opt: &mut cli::Opt) {
     opt.computed.true_color = match opt.true_color.as_ref() {
         "always" => true,
         "never" => false,
-        "auto" => is_truecolor_terminal(),
+        "auto" => is_truecolor_terminal(&opt.env),
         _ => {
             fatal(format!(
                 "Invalid value for --true-color option: {} (valid values are \"always\", \"never\", and \"auto\")",
@@ -650,8 +650,9 @@ fn set_true_color(opt: &mut cli::Opt) {
     };
 }
 
-fn is_truecolor_terminal() -> bool {
-    env::get_env_var("COLORTERM")
+fn is_truecolor_terminal(env: &DeltaEnv) -> bool {
+    env.colorterm
+        .as_ref()
         .map(|colorterm| colorterm == "truecolor" || colorterm == "24bit")
         .unwrap_or(false)
 }
@@ -663,15 +664,6 @@ fn set_git_config_entries(opt: &mut cli::Opt, git_config: &mut GitConfig) {
                 .insert(key.to_string(), GitConfigEntry::Style(style_string));
         }
     }
-
-    if let Some(repo) = &git_config.repo {
-        if let Some(workdir) = repo.workdir() {
-            opt.git_config_entries.insert(
-                "delta.__workdir__".to_string(),
-                GitConfigEntry::Path(workdir.to_path_buf()),
-            );
-        }
-    }
 }
 
 #[cfg(test)]
@@ -701,6 +693,7 @@ pub mod tests {
     file-modified-label = xxxyyyzzz
     file-removed-label = xxxyyyzzz
     file-renamed-label = xxxyyyzzz
+    file-transformation = s/foo/bar/
     right-arrow = xxxyyyzzz
     file-style = black black
     hunk-header-decoration-style = black black
@@ -768,6 +761,7 @@ pub mod tests {
         assert_eq!(opt.file_renamed_label, "xxxyyyzzz");
         assert_eq!(opt.right_arrow, "xxxyyyzzz");
         assert_eq!(opt.file_style, "black black");
+        assert_eq!(opt.file_regex_replacement, Some("s/foo/bar/".to_string()));
         assert_eq!(opt.hunk_header_decoration_style, "black black");
         assert_eq!(opt.hunk_header_style, "black black");
         assert_eq!(opt.keep_plus_minus_markers, true);
